from __future__ import print_function
from functools import partial

# Use the builtin 'input' in python2 and 'raw_input' in python3
try:
    input = raw_input
except:
    pass


def clear():
    import os
    os.system('cls' if os.name == 'nt' else 'clear')


def kernel_name(v):
    first = None
    if v[1] == "residual":
        first = "alpha"
    if v[1] == "jacobian":
        first = "jacobian"
    assert first

    second = None
    if v[0] == "cell":
        second = "volume"
    if v[0] == "exterior_facet":
        second = "boundary"
    if v[0] == "interior_facet":
        second = "skeleton"
    assert second

    return "{}_{}".format(first, second)


def show_kernel(which, kernel):
    clear()
    print("Showing the loo.py kernel for {}:\n".format(kernel_name(which)))
    print(kernel)
    print("Press Return to return to the previous menu")
    input()
    return kernel


def choose_transformation(which, kernel):
    choice = None
    while choice != "q":
        clear()
        keymap = {}
        print("Choose one of the following transformations to apply to {}:\n".format(kernel_name(which)))

        print("Transformations:")
        from dune.perftool.loopy.transformations import get_loopy_transformations
        for i, v in enumerate(get_loopy_transformations().values()):
            print("  {}) {}".format(chr(ord('a') + i), v.name))
            if v.description:
                print("       {}".format(v.description))
            keymap[chr(ord('a') + i)] = v

        print("\n  q) Return to kernel options")
        print("\nYour choice:")

        choice = input().lower()
        try:
            kernel = keymap[choice](kernel)
        except KeyError:
            pass

    return kernel


def show_code(which, kernel):
    clear()
    print("Showing the generated dune-pdelab code for {}:\n".format(kernel_name(which)))

    from dune.perftool.pdelab.localoperator import measure_specific_details, AssemblyMethod
    signature = measure_specific_details(which[0])["{}_signature".format(which[1])]
    print("".join(AssemblyMethod(signature, kernel).generate()))

    print("Press Return to return to the previous menu")
    input()
    return kernel


def optimize_kernel(which, kernels):
    kernel = kernels[which]
    choice = None

    while choice != "q":
        clear()
        print("Optimizing kernel {}:\n".format(kernel_name(which)))

        print("Available options:")
        print("  a) Show the loopy kernel")
        print("  b) Apply loopy transformation")
        print("  c) Show generated PDELab code for this kernel.")

        print("\n  q) Return to the kernel overview")
        print("\nYour choice:")

        choice = input().lower()
        try:
            kernel = {'a': partial(show_kernel, which),
                      'b': partial(choose_transformation, which),
                      'c': partial(show_code, which)
                     }[choice](kernel)
        except KeyError:
            pass

    kernels[which] = kernel


def kernel_choice(kernels):
    choice = None
    while choice != "q":
        clear()
        print("The following kernels are in the input. Pick one to optimize:")

        keymap = {}
        for i, k in enumerate(kernels.keys()):
            print("  {}) {}".format(chr(ord('a') + i), kernel_name(k)))
            keymap[chr(ord('a') + i)] = partial(optimize_kernel, k)

        print("\n  q) End this interactive session and proceed to code generation")

        print("\nYour choice: ")
        choice = input().lower()
        try:
            keymap[choice](kernels)
        except KeyError:
            pass


def start_interactive_session(kernels):
    clear()
    print("Welcome to the dune-perftool interactive mode!\n")

    kernel_choice(kernels)
